@@ -1,14 +1,15 @@
 use std::borrow::Cow;
 use std::collections::BTreeMap;
 use std::fmt::{self, Debug};
-use std::io::{self, Read};
+use std::io::{self, Read, Write};
 use std::sync::{Arc, Mutex, MutexGuard};
 
-use zip::{read::ZipFile, ZipArchive, ZipWriter};
-
-use crate::file_revision::FileRevision;
+use zip::result::ZipResult;
+use zip::write::FileOptions;
+use zip::{CompressionMethod, ZipArchive, ZipWriter, read::ZipFile};
 
 pub use self::path::{VendoredPath, VendoredPathBuf};
+use crate::file_revision::FileRevision;
 
 mod path;
 
@@ -20,6 +21,7 @@ type LockedZipArchive<'a> = MutexGuard<'a, VendoredZipArchive>;
 ///
 /// "Files" in the `VendoredFileSystem` are read-only and immutable.
 /// Directories are supported, but symlinks and hardlinks cannot exist.
+#[derive(Clone)]
 pub struct VendoredFileSystem {
     inner: Arc<Mutex<VendoredZipArchive>>,
 }
@@ -39,12 +41,6 @@ impl VendoredFileSystem {
         })
     }
 
-    pub fn snapshot(&self) -> Self {
-        Self {
-            inner: Arc::clone(&self.inner),
-        }
-    }
-
     pub fn exists(&self, path: impl AsRef<VendoredPath>) -> bool {
         fn exists(fs: &VendoredFileSystem, path: &VendoredPath) -> bool {
             let normalized = NormalizedVendoredPath::from(path);
@@ -102,7 +98,16 @@ impl VendoredFileSystem {
         fn read_to_string(fs: &VendoredFileSystem, path: &VendoredPath) -> Result<String> {
             let mut archive = fs.lock_archive();
             let mut zip_file = archive.lookup_path(&NormalizedVendoredPath::from(path))?;
-            let mut buffer = String::new();
+
+            // Pre-allocate the buffer with the size specified in the ZIP file metadata
+            // because `read_to_string` passes `None` as the size hint.
+            // But let's not trust the zip file metadata (even though it's vendored)
+            // and limit it to a reasonable size.
+            let mut buffer = String::with_capacity(
+                usize::try_from(zip_file.size())
+                    .unwrap_or(usize::MAX)
+                    .min(10_000_000),
+            );
             zip_file.read_to_string(&mut buffer)?;
             Ok(buffer)
         }
@@ -167,13 +172,12 @@ impl Default for VendoredFileSystem {
 /// that users of the `VendoredFileSystem` could realistically need.
 /// For debugging purposes, however, we want to have all information
 /// available.
-#[allow(unused)]
+#[expect(unused)]
 #[derive(Debug)]
 struct ZipFileDebugInfo {
     crc32_hash: u32,
     compressed_size: u64,
     uncompressed_size: u64,
-    compression_method: zip::CompressionMethod,
     kind: FileType,
 }
 
@@ -183,7 +187,6 @@ impl<'a> From<ZipFile<'a>> for ZipFileDebugInfo {
             crc32_hash: value.crc32(),
             compressed_size: value.compressed_size(),
             uncompressed_size: value.size(),
-            compression_method: value.compression(),
             kind: if value.is_dir() {
                 FileType::Directory
             } else {
@@ -267,7 +270,7 @@ impl VendoredZipArchive {
 #[derive(Debug, Clone, PartialEq, Eq)]
 struct NormalizedVendoredPath<'a>(Cow<'a, str>);
 
-impl<'a> NormalizedVendoredPath<'a> {
+impl NormalizedVendoredPath<'_> {
     fn with_trailing_slash(self) -> Self {
         debug_assert!(!self.0.ends_with('/'));
         let mut data = self.0.into_owned();
@@ -337,69 +340,61 @@ impl<'a> From<&'a VendoredPath> for NormalizedVendoredPath<'a> {
     }
 }
 
+pub struct VendoredFileSystemBuilder {
+    writer: ZipWriter<io::Cursor<Vec<u8>>>,
+    compression_method: CompressionMethod,
+}
+
+impl VendoredFileSystemBuilder {
+    pub fn new(compression_method: CompressionMethod) -> Self {
+        let buffer = io::Cursor::new(Vec::new());
+
+        Self {
+            writer: ZipWriter::new(buffer),
+            compression_method,
+        }
+    }
+
+    pub fn add_file(
+        &mut self,
+        path: impl AsRef<VendoredPath>,
+        content: &str,
+    ) -> std::io::Result<()> {
+        self.writer
+            .start_file(path.as_ref().as_str(), self.options())?;
+        self.writer.write_all(content.as_bytes())
+    }
+
+    pub fn add_directory(&mut self, path: impl AsRef<VendoredPath>) -> ZipResult<()> {
+        self.writer
+            .add_directory(path.as_ref().as_str(), self.options())
+    }
+
+    pub fn finish(mut self) -> Result<VendoredFileSystem> {
+        let buffer = self.writer.finish()?;
+
+        VendoredFileSystem::new(buffer.into_inner())
+    }
+
+    fn options(&self) -> FileOptions {
+        FileOptions::default()
+            .compression_method(self.compression_method)
+            .unix_permissions(0o644)
+    }
+}
+
 #[cfg(test)]
 pub(crate) mod tests {
-    use std::io::Write;
 
     use insta::assert_snapshot;
-    use zip::result::ZipResult;
-    use zip::write::FileOptions;
-    use zip::{CompressionMethod, ZipWriter};
 
     use super::*;
 
     const FUNCTOOLS_CONTENTS: &str = "def update_wrapper(): ...";
     const ASYNCIO_TASKS_CONTENTS: &str = "class Task: ...";
 
-    pub struct VendoredFileSystemBuilder {
-        writer: ZipWriter<io::Cursor<Vec<u8>>>,
-    }
-
-    impl Default for VendoredFileSystemBuilder {
-        fn default() -> Self {
-            Self::new()
-        }
-    }
-
-    impl VendoredFileSystemBuilder {
-        pub fn new() -> Self {
-            let buffer = io::Cursor::new(Vec::new());
-
-            Self {
-                writer: ZipWriter::new(buffer),
-            }
-        }
-
-        pub fn add_file(
-            &mut self,
-            path: impl AsRef<VendoredPath>,
-            content: &str,
-        ) -> std::io::Result<()> {
-            self.writer
-                .start_file(path.as_ref().as_str(), Self::options())?;
-            self.writer.write_all(content.as_bytes())
-        }
-
-        pub fn add_directory(&mut self, path: impl AsRef<VendoredPath>) -> ZipResult<()> {
-            self.writer
-                .add_directory(path.as_ref().as_str(), Self::options())
-        }
-
-        pub fn finish(mut self) -> Result<VendoredFileSystem> {
-            let buffer = self.writer.finish()?;
-
-            VendoredFileSystem::new(buffer.into_inner())
-        }
-
-        fn options() -> FileOptions {
-            FileOptions::default()
-                .compression_method(CompressionMethod::Zstd)
-                .unix_permissions(0o644)
-        }
-    }
-
     fn mock_typeshed() -> VendoredFileSystem {
-        let mut builder = VendoredFileSystemBuilder::new();
+        let mut builder = VendoredFileSystemBuilder::new(CompressionMethod::Stored);
 
         builder.add_directory("stdlib/").unwrap();
         builder
@@ -423,7 +418,7 @@ pub(crate) mod tests {
 
     #[test]
     fn filesystem_debug_implementation_alternate() {
-        assert_snapshot!(format!("{:#?}", mock_typeshed()), @r###"
+        assert_snapshot!(format!("{:#?}", mock_typeshed()), @r#"
         VendoredFileSystem {
             inner_mutex_poisoned: false,
             paths: [
@@ -437,33 +432,29 @@ pub(crate) mod tests {
                     crc32_hash: 0,
                     compressed_size: 0,
                     uncompressed_size: 0,
-                    compression_method: Stored,
                     kind: Directory,
                 },
                 "stdlib/asyncio/": ZipFileDebugInfo {
                     crc32_hash: 0,
                     compressed_size: 0,
                     uncompressed_size: 0,
-                    compression_method: Stored,
                     kind: Directory,
                 },
                 "stdlib/asyncio/tasks.pyi": ZipFileDebugInfo {
                     crc32_hash: 2826547428,
-                    compressed_size: 24,
+                    compressed_size: 15,
                     uncompressed_size: 15,
-                    compression_method: Zstd,
                     kind: File,
                 },
                 "stdlib/functools.pyi": ZipFileDebugInfo {
                     crc32_hash: 1099005079,
-                    compressed_size: 34,
+                    compressed_size: 25,
                     uncompressed_size: 25,
-                    compression_method: Zstd,
                     kind: File,
                 },
             },
         }
-        "###);
+        "#);
     }
 
     fn test_directory(dirname: &str) {
@@ -512,9 +503,11 @@ pub(crate) mod tests {
         let path = VendoredPath::new(path);
         assert!(!mock_typeshed.exists(path));
         assert!(mock_typeshed.metadata(path).is_err());
-        assert!(mock_typeshed
-            .read_to_string(path)
-            .is_err_and(|err| err.to_string().contains("file not found")));
+        assert!(
+            mock_typeshed
+                .read_to_string(path)
+                .is_err_and(|err| err.to_string().contains("file not found"))
+        );
     }
 
     #[test]
