@@ -67,3 +67,10 @@ print(fr""" ​
 """)
 print(Rf""" ​
 """)
+
+# https://github.com/astral-sh/ruff/issues/18815
+b = "\"
+sub = "\"
+esc = "\"
+zwsp = "\​"
+nul = "\ "
