@@ -1,21 +1,21 @@
+pub(crate) use async_busy_wait::*;
 pub(crate) use async_function_with_timeout::*;
+pub(crate) use async_zero_sleep::*;
 pub(crate) use blocking_http_call::*;
 pub(crate) use blocking_open_call::*;
 pub(crate) use blocking_process_invocation::*;
 pub(crate) use blocking_sleep::*;
-pub(crate) use sleep_forever_call::*;
+pub(crate) use cancel_scope_no_checkpoint::*;
+pub(crate) use long_sleep_not_forever::*;
 pub(crate) use sync_call::*;
-pub(crate) use timeout_without_await::*;
-pub(crate) use unneeded_sleep::*;
-pub(crate) use zero_sleep_call::*;
 
+mod async_busy_wait;
 mod async_function_with_timeout;
+mod async_zero_sleep;
 mod blocking_http_call;
 mod blocking_open_call;
 mod blocking_process_invocation;
 mod blocking_sleep;
-mod sleep_forever_call;
+mod cancel_scope_no_checkpoint;
+mod long_sleep_not_forever;
 mod sync_call;
-mod timeout_without_await;
-mod unneeded_sleep;
-mod zero_sleep_call;
