@@ -10,17 +10,20 @@ mod tests {
     use anyhow::Result;
     use test_case::test_case;
 
-    use crate::assert_messages;
+    use crate::assert_diagnostics;
     use crate::registry::Rule;
 
     use crate::settings::LinterSettings;
     use crate::test::test_path;
 
+    use ruff_python_ast::PythonVersion;
+
     #[test_case(Rule::AbstractBaseClassWithoutAbstractMethod, Path::new("B024.py"))]
     #[test_case(Rule::AssertFalse, Path::new("B011.py"))]
     #[test_case(Rule::AssertRaisesException, Path::new("B017.py"))]
     #[test_case(Rule::AssignmentToOsEnviron, Path::new("B003.py"))]
     #[test_case(Rule::CachedInstanceMethod, Path::new("B019.py"))]
+    #[test_case(Rule::ClassAsDataStructure, Path::new("class_as_data_structure.py"))]
     #[test_case(Rule::DuplicateHandlerException, Path::new("B014.py"))]
     #[test_case(Rule::DuplicateTryBlockException, Path::new("B025.py"))]
     #[test_case(Rule::DuplicateValue, Path::new("B033.py"))]
@@ -43,6 +46,7 @@ mod tests {
     #[test_case(Rule::MutableArgumentDefault, Path::new("B006_7.py"))]
     #[test_case(Rule::MutableArgumentDefault, Path::new("B006_8.py"))]
     #[test_case(Rule::MutableArgumentDefault, Path::new("B006_B008.py"))]
+    #[test_case(Rule::MutableArgumentDefault, Path::new("B006_1.pyi"))]
     #[test_case(Rule::NoExplicitStacklevel, Path::new("B028.py"))]
     #[test_case(Rule::RaiseLiteral, Path::new("B016.py"))]
     #[test_case(Rule::RaiseWithoutFromInsideExcept, Path::new("B904.py"))]
@@ -64,13 +68,43 @@ mod tests {
     #[test_case(Rule::UselessExpression, Path::new("B018.py"))]
     #[test_case(Rule::ReturnInGenerator, Path::new("B901.py"))]
     #[test_case(Rule::LoopIteratorMutation, Path::new("B909.py"))]
+    #[test_case(Rule::MutableContextvarDefault, Path::new("B039.py"))]
+    #[test_case(Rule::BatchedWithoutExplicitStrict, Path::new("B911.py"))]
     fn rules(rule_code: Rule, path: &Path) -> Result<()> {
         let snapshot = format!("{}_{}", rule_code.noqa_code(), path.to_string_lossy());
         let diagnostics = test_path(
             Path::new("flake8_bugbear").join(path).as_path(),
             &LinterSettings::for_rule(rule_code),
         )?;
-        assert_messages!(snapshot, diagnostics);
+        assert_diagnostics!(snapshot, diagnostics);
+        Ok(())
+    }
+
+    #[test_case(
+        Rule::ClassAsDataStructure,
+        Path::new("class_as_data_structure.py"),
+        PythonVersion::PY39
+    )]
+    fn rules_with_target_version(
+        rule_code: Rule,
+        path: &Path,
+        target_version: PythonVersion,
+    ) -> Result<()> {
+        let snapshot = format!(
+            "{}_py{}{}_{}",
+            rule_code.noqa_code(),
+            target_version.major,
+            target_version.minor,
+            path.to_string_lossy(),
+        );
+        let diagnostics = test_path(
+            Path::new("flake8_bugbear").join(path).as_path(),
+            &LinterSettings {
+                unresolved_target_version: target_version.into(),
+                ..LinterSettings::for_rule(rule_code)
+            },
+        )?;
+        assert_diagnostics!(snapshot, diagnostics);
         Ok(())
     }
 
@@ -81,7 +115,7 @@ mod tests {
             Path::new("flake8_bugbear").join(snapshot).as_path(),
             &LinterSettings::for_rule(Rule::ZipWithoutExplicitStrict),
         )?;
-        assert_messages!(snapshot, diagnostics);
+        assert_diagnostics!(snapshot, diagnostics);
         Ok(())
     }
 
@@ -100,7 +134,7 @@ mod tests {
                 ..LinterSettings::for_rule(Rule::MutableArgumentDefault)
             },
         )?;
-        assert_messages!(snapshot, diagnostics);
+        assert_diagnostics!(snapshot, diagnostics);
         Ok(())
     }
 
@@ -121,7 +155,23 @@ mod tests {
                 ..LinterSettings::for_rule(Rule::FunctionCallInDefaultArgument)
             },
         )?;
-        assert_messages!(snapshot, diagnostics);
+        assert_diagnostics!(snapshot, diagnostics);
+        Ok(())
+    }
+
+    #[test]
+    fn extend_mutable_contextvar_default() -> Result<()> {
+        let snapshot = "extend_mutable_contextvar_default".to_string();
+        let diagnostics = test_path(
+            Path::new("flake8_bugbear/B039_extended.py"),
+            &LinterSettings {
+                flake8_bugbear: super::settings::Settings {
+                    extend_immutable_calls: vec!["fastapi.Query".to_string()],
+                },
+                ..LinterSettings::for_rule(Rule::MutableContextvarDefault)
+            },
+        )?;
+        assert_diagnostics!(snapshot, diagnostics);
         Ok(())
     }
 }
