@@ -1,11 +1,13 @@
-use ruff_diagnostics::{AlwaysFixableViolation, Violation};
-use ruff_diagnostics::{Diagnostic, Edit, Fix};
-use ruff_macros::{derive_message_formats, violation};
+use ruff_macros::{ViolationMetadata, derive_message_formats};
 use ruff_python_index::Indexer;
 use ruff_python_parser::{TokenKind, Tokens};
-use ruff_source_file::Locator;
 use ruff_text_size::{Ranged, TextRange};
 
+use crate::Locator;
+use crate::checkers::ast::LintContext;
+use crate::{AlwaysFixableViolation, Violation};
+use crate::{Edit, Fix};
+
 /// Simplified token type.
 #[derive(Copy, Clone, PartialEq, Eq)]
 enum TokenType {
@@ -73,7 +75,7 @@ impl From<(TokenKind, TextRange)> for SimpleToken {
             TokenKind::Import => TokenType::Named,
             _ => TokenType::Irrelevant,
         };
-        #[allow(clippy::inconsistent_struct_constructor)]
+        #[expect(clippy::inconsistent_struct_constructor)]
         Self { range, ty }
     }
 }
@@ -138,13 +140,19 @@ impl Context {
 ///     "baz": 2,
 /// }
 /// ```
-#[violation]
-pub struct MissingTrailingComma;
+///
+/// ## Formatter compatibility
+/// We recommend against using this rule alongside the [formatter]. The
+/// formatter enforces consistent use of trailing commas, making the rule redundant.
+///
+/// [formatter]:https://docs.astral.sh/ruff/formatter/
+#[derive(ViolationMetadata)]
+pub(crate) struct MissingTrailingComma;
 
 impl AlwaysFixableViolation for MissingTrailingComma {
     #[derive_message_formats]
     fn message(&self) -> String {
-        format!("Trailing comma missing")
+        "Trailing comma missing".to_string()
     }
 
     fn fix_title(&self) -> String {
@@ -183,13 +191,13 @@ impl AlwaysFixableViolation for MissingTrailingComma {
 ///
 /// foo = (json.dumps({"bar": 1}),)
 /// ```
-#[violation]
-pub struct TrailingCommaOnBareTuple;
+#[derive(ViolationMetadata)]
+pub(crate) struct TrailingCommaOnBareTuple;
 
 impl Violation for TrailingCommaOnBareTuple {
     #[derive_message_formats]
     fn message(&self) -> String {
-        format!("Trailing comma on bare tuple prohibited")
+        "Trailing comma on bare tuple prohibited".to_string()
     }
 }
 
@@ -209,13 +217,19 @@ impl Violation for TrailingCommaOnBareTuple {
 /// ```python
 /// foo = (1, 2, 3)
 /// ```
-#[violation]
-pub struct ProhibitedTrailingComma;
+///
+/// ## Formatter compatibility
+/// We recommend against using this rule alongside the [formatter]. The
+/// formatter enforces consistent use of trailing commas, making the rule redundant.
+///
+/// [formatter]:https://docs.astral.sh/ruff/formatter/
+#[derive(ViolationMetadata)]
+pub(crate) struct ProhibitedTrailingComma;
 
 impl AlwaysFixableViolation for ProhibitedTrailingComma {
     #[derive_message_formats]
     fn message(&self) -> String {
-        format!("Trailing comma prohibited")
+        "Trailing comma prohibited".to_string()
     }
 
     fn fix_title(&self) -> String {
@@ -225,13 +239,13 @@ impl AlwaysFixableViolation for ProhibitedTrailingComma {
 
 /// COM812, COM818, COM819
 pub(crate) fn trailing_commas(
-    diagnostics: &mut Vec<Diagnostic>,
+    lint_context: &LintContext,
     tokens: &Tokens,
     locator: &Locator,
     indexer: &Indexer,
 ) {
     let mut fstrings = 0u32;
-    let simple_tokens = tokens.up_to_first_unknown().iter().filter_map(|token| {
+    let simple_tokens = tokens.iter().filter_map(|token| {
         match token.kind() {
             // Completely ignore comments -- they just interfere with the logic.
             TokenKind::Comment => None,
@@ -278,9 +292,7 @@ pub(crate) fn trailing_commas(
         // Update the comma context stack.
         let context = update_context(token, prev, prev_prev, &mut stack);
 
-        if let Some(diagnostic) = check_token(token, prev, prev_prev, context, locator) {
-            diagnostics.push(diagnostic);
-        }
+        check_token(token, prev, prev_prev, context, locator, lint_context);
 
         // Pop the current context if the current token ended it.
         // The top context is never popped (if unbalanced closing brackets).
@@ -306,7 +318,8 @@ fn check_token(
     prev_prev: SimpleToken,
     context: Context,
     locator: &Locator,
-) -> Option<Diagnostic> {
+    lint_context: &LintContext,
+) {
     // Is it allowed to have a trailing comma before this token?
     let comma_allowed = token.ty == TokenType::ClosingBracket
         && match context.ty {
@@ -339,20 +352,25 @@ fn check_token(
     };
 
     if comma_prohibited {
-        let mut diagnostic = Diagnostic::new(ProhibitedTrailingComma, prev.range());
-        diagnostic.set_fix(Fix::safe_edit(Edit::range_deletion(diagnostic.range())));
-        return Some(diagnostic);
+        if let Some(mut diagnostic) =
+            lint_context.report_diagnostic_if_enabled(ProhibitedTrailingComma, prev.range())
+        {
+            let range = diagnostic.range();
+            diagnostic.set_fix(Fix::safe_edit(Edit::range_deletion(range)));
+            return;
+        }
     }
 
     // Is prev a prohibited trailing comma on a bare tuple?
     // Approximation: any comma followed by a statement-ending newline.
     let bare_comma_prohibited = prev.ty == TokenType::Comma && token.ty == TokenType::Newline;
     if bare_comma_prohibited {
-        return Some(Diagnostic::new(TrailingCommaOnBareTuple, prev.range()));
+        lint_context.report_diagnostic_if_enabled(TrailingCommaOnBareTuple, prev.range());
+        return;
     }
 
     if !comma_allowed {
-        return None;
+        return;
     }
 
     // Comma is required if:
@@ -369,20 +387,19 @@ fn check_token(
                 | TokenType::OpeningCurlyBracket
         );
     if comma_required {
-        let mut diagnostic =
-            Diagnostic::new(MissingTrailingComma, TextRange::empty(prev_prev.end()));
-        // Create a replacement that includes the final bracket (or other token),
-        // rather than just inserting a comma at the end. This prevents the UP034 fix
-        // removing any brackets in the same linter pass - doing both at the same time could
-        // lead to a syntax error.
-        let contents = locator.slice(prev_prev.range());
-        diagnostic.set_fix(Fix::safe_edit(Edit::range_replacement(
-            format!("{contents},"),
-            prev_prev.range(),
-        )));
-        Some(diagnostic)
-    } else {
-        None
+        if let Some(mut diagnostic) = lint_context
+            .report_diagnostic_if_enabled(MissingTrailingComma, TextRange::empty(prev_prev.end()))
+        {
+            // Create a replacement that includes the final bracket (or other token),
+            // rather than just inserting a comma at the end. This prevents the UP034 fix
+            // removing any brackets in the same linter pass - doing both at the same time could
+            // lead to a syntax error.
+            let contents = locator.slice(prev_prev.range());
+            diagnostic.set_fix(Fix::safe_edit(Edit::range_replacement(
+                format!("{contents},"),
+                prev_prev.range(),
+            )));
+        }
     }
 }
 
