@@ -1,4 +1,3 @@
-use ruff_diagnostics::{Diagnostic, Edit, Fix};
 use ruff_python_ast::{self as ast, Arguments, Expr, Keyword, Operator};
 use ruff_python_semantic::analyze::logging;
 use ruff_python_stdlib::logging::LoggingLevel;
@@ -10,6 +9,7 @@ use crate::rules::flake8_logging_format::violations::{
     LoggingExcInfo, LoggingExtraAttrClash, LoggingFString, LoggingPercentFormat,
     LoggingRedundantExcInfo, LoggingStringConcat, LoggingStringFormat, LoggingWarn,
 };
+use crate::{Edit, Fix};
 
 /// Returns `true` if the attribute is a reserved attribute on the `logging` module's `LogRecord`
 /// class.
@@ -42,42 +42,28 @@ fn is_reserved_attr(attr: &str) -> bool {
 }
 
 /// Check logging messages for violations.
-fn check_msg(checker: &mut Checker, msg: &Expr) {
+fn check_msg(checker: &Checker, msg: &Expr) {
     match msg {
         // Check for string concatenation and percent format.
         Expr::BinOp(ast::ExprBinOp { op, .. }) => match op {
             Operator::Add => {
-                if checker.enabled(Rule::LoggingStringConcat) {
-                    checker
-                        .diagnostics
-                        .push(Diagnostic::new(LoggingStringConcat, msg.range()));
-                }
+                checker.report_diagnostic_if_enabled(LoggingStringConcat, msg.range());
             }
             Operator::Mod => {
-                if checker.enabled(Rule::LoggingPercentFormat) {
-                    checker
-                        .diagnostics
-                        .push(Diagnostic::new(LoggingPercentFormat, msg.range()));
-                }
+                checker.report_diagnostic_if_enabled(LoggingPercentFormat, msg.range());
             }
             _ => {}
         },
         // Check for f-strings.
         Expr::FString(_) => {
-            if checker.enabled(Rule::LoggingFString) {
-                checker
-                    .diagnostics
-                    .push(Diagnostic::new(LoggingFString, msg.range()));
-            }
+            checker.report_diagnostic_if_enabled(LoggingFString, msg.range());
         }
         // Check for .format() calls.
         Expr::Call(ast::ExprCall { func, .. }) => {
-            if checker.enabled(Rule::LoggingStringFormat) {
+            if checker.is_rule_enabled(Rule::LoggingStringFormat) {
                 if let Expr::Attribute(ast::ExprAttribute { value, attr, .. }) = func.as_ref() {
                     if attr == "format" && value.is_literal_expr() {
-                        checker
-                            .diagnostics
-                            .push(Diagnostic::new(LoggingStringFormat, msg.range()));
+                        checker.report_diagnostic(LoggingStringFormat, msg.range());
                     }
                 }
             }
@@ -87,7 +73,7 @@ fn check_msg(checker: &mut Checker, msg: &Expr) {
 }
 
 /// Check contents of the `extra` argument to logging calls.
-fn check_log_record_attr_clash(checker: &mut Checker, extra: &Keyword) {
+fn check_log_record_attr_clash(checker: &Checker, extra: &Keyword) {
     match &extra.value {
         Expr::Dict(dict) => {
             for invalid_key in dict.iter_keys().filter_map(|key| {
@@ -98,10 +84,10 @@ fn check_log_record_attr_clash(checker: &mut Checker, extra: &Keyword) {
                     None
                 }
             }) {
-                checker.diagnostics.push(Diagnostic::new(
+                checker.report_diagnostic(
                     LoggingExtraAttrClash(invalid_key.value.to_string()),
                     invalid_key.range(),
-                ));
+                );
             }
         }
         Expr::Call(ast::ExprCall {
@@ -110,13 +96,13 @@ fn check_log_record_attr_clash(checker: &mut Checker, extra: &Keyword) {
             ..
         }) => {
             if checker.semantic().match_builtin_expr(func, "dict") {
-                for keyword in keywords.iter() {
+                for keyword in keywords {
                     if let Some(attr) = &keyword.arg {
                         if is_reserved_attr(attr) {
-                            checker.diagnostics.push(Diagnostic::new(
+                            checker.report_diagnostic(
                                 LoggingExtraAttrClash(attr.to_string()),
                                 keyword.range(),
-                            ));
+                            );
                         }
                     }
                 }
@@ -145,7 +131,7 @@ impl LoggingCallType {
 }
 
 /// Check logging calls for violations.
-pub(crate) fn logging_call(checker: &mut Checker, call: &ast::ExprCall) {
+pub(crate) fn logging_call(checker: &Checker, call: &ast::ExprCall) {
     // Determine the call type (e.g., `info` vs. `exception`) and the range of the attribute.
     let (logging_call_type, range) = match call.func.as_ref() {
         Expr::Attribute(ast::ExprAttribute { value: _, attr, .. }) => {
@@ -155,7 +141,7 @@ pub(crate) fn logging_call(checker: &mut Checker, call: &ast::ExprCall) {
             if !logging::is_logger_candidate(
                 &call.func,
                 checker.semantic(),
-                &checker.settings.logger_objects,
+                &checker.settings().logger_objects,
             ) {
                 return;
             }
@@ -179,36 +165,35 @@ pub(crate) fn logging_call(checker: &mut Checker, call: &ast::ExprCall) {
         _ => return,
     };
 
-    // G001 - G004
+    // G001, G002, G003, G004
     let msg_pos = usize::from(matches!(logging_call_type, LoggingCallType::LogCall));
-    if let Some(format_arg) = call.arguments.find_argument("msg", msg_pos) {
+    if let Some(format_arg) = call.arguments.find_argument_value("msg", msg_pos) {
         check_msg(checker, format_arg);
     }
 
     // G010
-    if checker.enabled(Rule::LoggingWarn) {
+    if checker.is_rule_enabled(Rule::LoggingWarn) {
         if matches!(
             logging_call_type,
             LoggingCallType::LevelCall(LoggingLevel::Warn)
         ) {
-            let mut diagnostic = Diagnostic::new(LoggingWarn, range);
+            let mut diagnostic = checker.report_diagnostic(LoggingWarn, range);
             diagnostic.set_fix(Fix::safe_edit(Edit::range_replacement(
                 "warning".to_string(),
                 range,
             )));
-            checker.diagnostics.push(diagnostic);
         }
     }
 
     // G101
-    if checker.enabled(Rule::LoggingExtraAttrClash) {
+    if checker.is_rule_enabled(Rule::LoggingExtraAttrClash) {
         if let Some(extra) = call.arguments.find_keyword("extra") {
             check_log_record_attr_clash(checker, extra);
         }
     }
 
     // G201, G202
-    if checker.any_enabled(&[Rule::LoggingExcInfo, Rule::LoggingRedundantExcInfo]) {
+    if checker.any_rule_enabled(&[Rule::LoggingExcInfo, Rule::LoggingRedundantExcInfo]) {
         if !checker.semantic().in_exception_handler() {
             return;
         }
@@ -218,18 +203,10 @@ pub(crate) fn logging_call(checker: &mut Checker, call: &ast::ExprCall) {
         if let LoggingCallType::LevelCall(logging_level) = logging_call_type {
             match logging_level {
                 LoggingLevel::Error => {
-                    if checker.enabled(Rule::LoggingExcInfo) {
-                        checker
-                            .diagnostics
-                            .push(Diagnostic::new(LoggingExcInfo, range));
-                    }
+                    checker.report_diagnostic_if_enabled(LoggingExcInfo, range);
                 }
                 LoggingLevel::Exception => {
-                    if checker.enabled(Rule::LoggingRedundantExcInfo) {
-                        checker
-                            .diagnostics
-                            .push(Diagnostic::new(LoggingRedundantExcInfo, exc_info.range()));
-                    }
+                    checker.report_diagnostic_if_enabled(LoggingRedundantExcInfo, exc_info.range());
                 }
                 _ => {}
             }
