@@ -13,8 +13,8 @@ use crate::checkers::ast::Checker;
 use crate::registry::Rule;
 use crate::{Edit, Fix, FixAvailability, Violation};
 
-use super::super::types;
-use super::helpers::{is_pytest_parametrize, split_names};
+use crate::rules::flake8_pytest_style::helpers::{is_pytest_parametrize, split_names};
+use crate::rules::flake8_pytest_style::types;
 
 /// ## What it does
 /// Checks for the type of parameter names passed to `pytest.mark.parametrize`.
@@ -335,7 +335,10 @@ fn get_parametrize_name_range(
 
 /// PT006
 fn check_names(checker: &Checker, call: &ExprCall, expr: &Expr, argvalues: &Expr) {
-    let names_type = checker.settings.flake8_pytest_style.parametrize_names_type;
+    let names_type = checker
+        .settings()
+        .flake8_pytest_style
+        .parametrize_names_type;
 
     match expr {
         Expr::StringLiteral(ast::ExprStringLiteral { value, .. }) => {
@@ -516,10 +519,13 @@ fn check_names(checker: &Checker, call: &ExprCall, expr: &Expr, argvalues: &Expr
 
 /// PT007
 fn check_values(checker: &Checker, names: &Expr, values: &Expr) {
-    let values_type = checker.settings.flake8_pytest_style.parametrize_values_type;
+    let values_type = checker
+        .settings()
+        .flake8_pytest_style
+        .parametrize_values_type;
 
     let values_row_type = checker
-        .settings
+        .settings()
         .flake8_pytest_style
         .parametrize_values_row_type;
 
@@ -865,7 +871,7 @@ pub(crate) fn parametrize(checker: &Checker, call: &ExprCall) {
         return;
     }
 
-    if checker.enabled(Rule::PytestParametrizeNamesWrongType) {
+    if checker.is_rule_enabled(Rule::PytestParametrizeNamesWrongType) {
         let names = call.arguments.find_argument_value("argnames", 0);
         let values = call.arguments.find_argument_value("argvalues", 1);
 
@@ -873,7 +879,7 @@ pub(crate) fn parametrize(checker: &Checker, call: &ExprCall) {
             check_names(checker, call, names, values);
         }
     }
-    if checker.enabled(Rule::PytestParametrizeValuesWrongType) {
+    if checker.is_rule_enabled(Rule::PytestParametrizeValuesWrongType) {
         let names = call.arguments.find_argument_value("argnames", 0);
         let values = call.arguments.find_argument_value("argvalues", 1);
 
@@ -881,7 +887,7 @@ pub(crate) fn parametrize(checker: &Checker, call: &ExprCall) {
             check_values(checker, names, values);
         }
     }
-    if checker.enabled(Rule::PytestDuplicateParametrizeTestCases) {
+    if checker.is_rule_enabled(Rule::PytestDuplicateParametrizeTestCases) {
         if let Some(values) = call.arguments.find_argument_value("argvalues", 1) {
             check_duplicates(checker, values);
         }
