@@ -3,20 +3,22 @@ use std::borrow::Cow;
 use anyhow::Result;
 use rustc_hash::FxHashMap;
 
-use ruff_diagnostics::{Diagnostic, DiagnosticKind, Fix, FixAvailability, Violation};
-use ruff_macros::{derive_message_formats, violation};
+use ruff_macros::{ViolationMetadata, derive_message_formats};
 use ruff_python_semantic::{Binding, Imported, NodeId, Scope};
-use ruff_text_size::Ranged;
+use ruff_text_size::{Ranged, TextRange};
 
-use crate::checkers::ast::Checker;
+use crate::checkers::ast::{Checker, DiagnosticGuard};
 use crate::codes::Rule;
 use crate::fix;
 use crate::importer::ImportedMembers;
+use crate::preview::is_full_path_match_source_strategy_enabled;
 use crate::rules::flake8_type_checking::helpers::{
     filter_contained, is_typing_reference, quote_annotation,
 };
 use crate::rules::flake8_type_checking::imports::ImportBinding;
-use crate::rules::isort::{categorize, ImportSection, ImportType};
+use crate::rules::isort::categorize::MatchSourceStrategy;
+use crate::rules::isort::{ImportSection, ImportType, categorize};
+use crate::{Fix, FixAvailability, Violation};
 
 /// ## What it does
 /// Checks for first-party imports that are only used for type annotations, but
@@ -63,16 +65,23 @@ use crate::rules::isort::{categorize, ImportSection, ImportType};
 ///     return len(sized)
 /// ```
 ///
+///
+/// ## Preview
+/// When [preview](https://docs.astral.sh/ruff/preview/) is enabled,
+/// the criterion for determining whether an import is first-party
+/// is stricter, which could affect whether this lint is triggered vs [`TC001`](https://docs.astral.sh/ruff/rules/typing-only-third-party-import/). See [this FAQ section](https://docs.astral.sh/ruff/faq/#how-does-ruff-determine-which-of-my-imports-are-first-party-third-party-etc) for more details.
+///
 /// ## Options
 /// - `lint.flake8-type-checking.quote-annotations`
 /// - `lint.flake8-type-checking.runtime-evaluated-base-classes`
 /// - `lint.flake8-type-checking.runtime-evaluated-decorators`
+/// - `lint.flake8-type-checking.strict`
 /// - `lint.typing-modules`
 ///
 /// ## References
-/// - [PEP 536](https://peps.python.org/pep-0563/#runtime-annotation-resolution-and-type-checking)
-#[violation]
-pub struct TypingOnlyFirstPartyImport {
+/// - [PEP 563: Runtime annotation resolution and `TYPE_CHECKING`](https://peps.python.org/pep-0563/#runtime-annotation-resolution-and-type-checking)
+#[derive(ViolationMetadata)]
+pub(crate) struct TypingOnlyFirstPartyImport {
     qualified_name: String,
 }
 
@@ -137,16 +146,22 @@ impl Violation for TypingOnlyFirstPartyImport {
 ///     return len(df)
 /// ```
 ///
+/// ## Preview
+/// When [preview](https://docs.astral.sh/ruff/preview/) is enabled,
+/// the criterion for determining whether an import is first-party
+/// is stricter, which could affect whether this lint is triggered vs [`TC001`](https://docs.astral.sh/ruff/rules/typing-only-first-party-import/). See [this FAQ section](https://docs.astral.sh/ruff/faq/#how-does-ruff-determine-which-of-my-imports-are-first-party-third-party-etc) for more details.
+///
 /// ## Options
 /// - `lint.flake8-type-checking.quote-annotations`
 /// - `lint.flake8-type-checking.runtime-evaluated-base-classes`
 /// - `lint.flake8-type-checking.runtime-evaluated-decorators`
+/// - `lint.flake8-type-checking.strict`
 /// - `lint.typing-modules`
 ///
 /// ## References
-/// - [PEP 536](https://peps.python.org/pep-0563/#runtime-annotation-resolution-and-type-checking)
-#[violation]
-pub struct TypingOnlyThirdPartyImport {
+/// - [PEP 563: Runtime annotation resolution and `TYPE_CHECKING`](https://peps.python.org/pep-0563/#runtime-annotation-resolution-and-type-checking)
+#[derive(ViolationMetadata)]
+pub(crate) struct TypingOnlyThirdPartyImport {
     qualified_name: String,
 }
 
@@ -215,12 +230,13 @@ impl Violation for TypingOnlyThirdPartyImport {
 /// - `lint.flake8-type-checking.quote-annotations`
 /// - `lint.flake8-type-checking.runtime-evaluated-base-classes`
 /// - `lint.flake8-type-checking.runtime-evaluated-decorators`
+/// - `lint.flake8-type-checking.strict`
 /// - `lint.typing-modules`
 ///
 /// ## References
-/// - [PEP 536](https://peps.python.org/pep-0563/#runtime-annotation-resolution-and-type-checking)
-#[violation]
-pub struct TypingOnlyStandardLibraryImport {
+/// - [PEP 563: Runtime annotation resolution and `TYPE_CHECKING`](https://peps.python.org/pep-0563/#runtime-annotation-resolution-and-type-checking)
+#[derive(ViolationMetadata)]
+pub(crate) struct TypingOnlyStandardLibraryImport {
     qualified_name: String,
 }
 
@@ -240,12 +256,11 @@ impl Violation for TypingOnlyStandardLibraryImport {
     }
 }
 
-/// TCH001, TCH002, TCH003
+/// TC001, TC002, TC003
 pub(crate) fn typing_only_runtime_import(
     checker: &Checker,
     scope: &Scope,
     runtime_imports: &[&Binding],
-    diagnostics: &mut Vec<Diagnostic>,
 ) {
     // Collect all typing-only imports by statement and import type.
     let mut errors_by_statement: FxHashMap<(NodeId, ImportType), Vec<ImportBinding>> =
@@ -258,7 +273,7 @@ pub(crate) fn typing_only_runtime_import(
 
         // If we're in un-strict mode, don't flag typing-only imports that are
         // implicitly loaded by way of a valid runtime import.
-        if !checker.settings.flake8_type_checking.strict
+        if !checker.settings().flake8_type_checking.strict
             && runtime_imports
                 .iter()
                 .any(|import| is_implicit_import(binding, import))
@@ -279,7 +294,7 @@ pub(crate) fn typing_only_runtime_import(
                 .references()
                 .map(|reference_id| checker.semantic().reference(reference_id))
                 .all(|reference| {
-                    is_typing_reference(reference, &checker.settings.flake8_type_checking)
+                    is_typing_reference(reference, &checker.settings().flake8_type_checking)
                 })
         {
             let qualified_name = import.qualified_name();
@@ -287,7 +302,7 @@ pub(crate) fn typing_only_runtime_import(
             if is_exempt(
                 &qualified_name.to_string(),
                 &checker
-                    .settings
+                    .settings()
                     .flake8_type_checking
                     .exempt_modules
                     .iter()
@@ -297,18 +312,28 @@ pub(crate) fn typing_only_runtime_import(
                 continue;
             }
 
+            let source_name = import.source_name().join(".");
+
             // Categorize the import, using coarse-grained categorization.
+            let match_source_strategy =
+                if is_full_path_match_source_strategy_enabled(checker.settings()) {
+                    MatchSourceStrategy::FullPath
+                } else {
+                    MatchSourceStrategy::Root
+                };
+
             let import_type = match categorize(
-                &qualified_name.to_string(),
-                0,
-                &checker.settings.src,
+                &source_name,
+                qualified_name.is_unresolved_import(),
+                &checker.settings().src,
                 checker.package(),
-                checker.settings.isort.detect_same_package,
-                &checker.settings.isort.known_modules,
-                checker.settings.target_version,
-                checker.settings.isort.no_sections,
-                &checker.settings.isort.section_order,
-                &checker.settings.isort.default_section,
+                checker.settings().isort.detect_same_package,
+                &checker.settings().isort.known_modules,
+                checker.target_version(),
+                checker.settings().isort.no_sections,
+                &checker.settings().isort.section_order,
+                &checker.settings().isort.default_section,
+                match_source_strategy,
             ) {
                 ImportSection::Known(ImportType::LocalFolder | ImportType::FirstParty) => {
                     ImportType::FirstParty
@@ -322,7 +347,7 @@ pub(crate) fn typing_only_runtime_import(
                 }
             };
 
-            if !checker.enabled(rule_for(import_type)) {
+            if !checker.is_rule_enabled(rule_for(import_type)) {
                 continue;
             }
 
@@ -368,8 +393,10 @@ pub(crate) fn typing_only_runtime_import(
             ..
         } in imports
         {
-            let mut diagnostic = Diagnostic::new(
-                diagnostic_for(import_type, import.qualified_name().to_string()),
+            let mut diagnostic = diagnostic_for(
+                checker,
+                import_type,
+                import.qualified_name().to_string(),
                 range,
             );
             if let Some(range) = parent_range {
@@ -378,7 +405,6 @@ pub(crate) fn typing_only_runtime_import(
             if let Some(fix) = fix.as_ref() {
                 diagnostic.set_fix(fix.clone());
             }
-            diagnostics.push(diagnostic);
         }
     }
 
@@ -392,14 +418,15 @@ pub(crate) fn typing_only_runtime_import(
             ..
         } in imports
         {
-            let mut diagnostic = Diagnostic::new(
-                diagnostic_for(import_type, import.qualified_name().to_string()),
+            let mut diagnostic = diagnostic_for(
+                checker,
+                import_type,
+                import.qualified_name().to_string(),
                 range,
             );
             if let Some(range) = parent_range {
                 diagnostic.set_parent(range.start());
             }
-            diagnostics.push(diagnostic);
         }
     }
 }
@@ -415,11 +442,22 @@ fn rule_for(import_type: ImportType) -> Rule {
 }
 
 /// Return the [`Diagnostic`] for the given import type.
-fn diagnostic_for(import_type: ImportType, qualified_name: String) -> DiagnosticKind {
+fn diagnostic_for<'a, 'b>(
+    checker: &'a Checker<'b>,
+    import_type: ImportType,
+    qualified_name: String,
+    range: TextRange,
+) -> DiagnosticGuard<'a, 'b> {
     match import_type {
-        ImportType::StandardLibrary => TypingOnlyStandardLibraryImport { qualified_name }.into(),
-        ImportType::ThirdParty => TypingOnlyThirdPartyImport { qualified_name }.into(),
-        ImportType::FirstParty => TypingOnlyFirstPartyImport { qualified_name }.into(),
+        ImportType::StandardLibrary => {
+            checker.report_diagnostic(TypingOnlyStandardLibraryImport { qualified_name }, range)
+        }
+        ImportType::ThirdParty => {
+            checker.report_diagnostic(TypingOnlyThirdPartyImport { qualified_name }, range)
+        }
+        ImportType::FirstParty => {
+            checker.report_diagnostic(TypingOnlyFirstPartyImport { qualified_name }, range)
+        }
         _ => unreachable!("Unexpected import type"),
     }
 }
@@ -505,16 +543,16 @@ fn fix_imports(checker: &Checker, node_id: NodeId, imports: &[ImportBinding]) ->
                         Some(quote_annotation(
                             reference.expression_id()?,
                             checker.semantic(),
-                            checker.locator(),
                             checker.stylist(),
-                            checker.generator(),
+                            checker.locator(),
+                            checker.default_string_flags(),
                         ))
                     } else {
                         None
                     }
                 })
             })
-            .collect::<Result<Vec<_>>>()?,
+            .collect::<Vec<_>>(),
     );
 
     Ok(Fix::unsafe_edits(
