@@ -1,5 +1,5 @@
 //! Rules from [Pylint](https://pypi.org/project/pylint/).
-mod helpers;
+pub(crate) mod helpers;
 pub(crate) mod rules;
 pub mod settings;
 
@@ -9,15 +9,16 @@ mod tests {
 
     use anyhow::Result;
     use regex::Regex;
+    use ruff_python_ast::PythonVersion;
     use rustc_hash::FxHashSet;
     use test_case::test_case;
 
     use crate::registry::Rule;
-    use crate::rules::pylint;
+    use crate::rules::{flake8_tidy_imports, pylint};
 
-    use crate::assert_messages;
-    use crate::settings::types::PythonVersion;
+    use crate::assert_diagnostics;
     use crate::settings::LinterSettings;
+    use crate::settings::types::PreviewMode;
     use crate::test::test_path;
 
     #[test_case(Rule::SingledispatchMethod, Path::new("singledispatch_method.py"))]
@@ -27,6 +28,7 @@ mod tests {
     )]
     #[test_case(Rule::AssertOnStringLiteral, Path::new("assert_on_string_literal.py"))]
     #[test_case(Rule::AwaitOutsideAsync, Path::new("await_outside_async.py"))]
+    #[test_case(Rule::AwaitOutsideAsync, Path::new("await_outside_async.ipynb"))]
     #[test_case(Rule::BadOpenMode, Path::new("bad_open_mode.py"))]
     #[test_case(
         Rule::BadStringFormatCharacter,
@@ -36,6 +38,10 @@ mod tests {
     #[test_case(Rule::BadStringFormatType, Path::new("bad_string_format_type.py"))]
     #[test_case(Rule::BidirectionalUnicode, Path::new("bidirectional_unicode.py"))]
     #[test_case(Rule::BinaryOpException, Path::new("binary_op_exception.py"))]
+    #[test_case(
+        Rule::BooleanChainedComparison,
+        Path::new("boolean_chained_comparison.py")
+    )]
     #[test_case(Rule::CollapsibleElseIf, Path::new("collapsible_else_if.py"))]
     #[test_case(Rule::CompareToEmptyString, Path::new("compare_to_empty_string.py"))]
     #[test_case(Rule::ComparisonOfConstant, Path::new("comparison_of_constant.py"))]
@@ -58,6 +64,10 @@ mod tests {
     #[test_case(Rule::SysExitAlias, Path::new("sys_exit_alias_10.py"))]
     #[test_case(Rule::SysExitAlias, Path::new("sys_exit_alias_11.py"))]
     #[test_case(Rule::SysExitAlias, Path::new("sys_exit_alias_12.py"))]
+    #[test_case(Rule::SysExitAlias, Path::new("sys_exit_alias_13.py"))]
+    #[test_case(Rule::SysExitAlias, Path::new("sys_exit_alias_14.py"))]
+    #[test_case(Rule::SysExitAlias, Path::new("sys_exit_alias_15.py"))]
+    #[test_case(Rule::SysExitAlias, Path::new("sys_exit_alias_16.py"))]
     #[test_case(Rule::ContinueInFinally, Path::new("continue_in_finally.py"))]
     #[test_case(Rule::GlobalStatement, Path::new("global_statement.py"))]
     #[test_case(
@@ -100,6 +110,7 @@ mod tests {
         Rule::InvalidCharacterBackspace,
         Path::new("invalid_characters_syntax_error.py")
     )]
+    #[test_case(Rule::ShallowCopyEnviron, Path::new("shallow_copy_environ.py"))]
     #[test_case(Rule::InvalidEnvvarDefault, Path::new("invalid_envvar_default.py"))]
     #[test_case(Rule::InvalidEnvvarValue, Path::new("invalid_envvar_value.py"))]
     #[test_case(Rule::IterationOverSet, Path::new("iteration_over_set.py"))]
@@ -112,6 +123,10 @@ mod tests {
         Path::new("named_expr_without_context.py")
     )]
     #[test_case(Rule::NonlocalAndGlobal, Path::new("nonlocal_and_global.py"))]
+    #[test_case(
+        Rule::RedefinedSlotsInSubclass,
+        Path::new("redefined_slots_in_subclass.py")
+    )]
     #[test_case(Rule::NonlocalWithoutBinding, Path::new("nonlocal_without_binding.py"))]
     #[test_case(Rule::NonSlotAssignment, Path::new("non_slot_assignment.py"))]
     #[test_case(Rule::PropertyWithParameters, Path::new("property_with_parameters.py"))]
@@ -123,7 +138,10 @@ mod tests {
     #[test_case(Rule::RedefinedLoopName, Path::new("redefined_loop_name.py"))]
     #[test_case(Rule::ReturnInInit, Path::new("return_in_init.py"))]
     #[test_case(Rule::TooManyArguments, Path::new("too_many_arguments.py"))]
-    #[test_case(Rule::TooManyPositional, Path::new("too_many_positional.py"))]
+    #[test_case(
+        Rule::TooManyPositionalArguments,
+        Path::new("too_many_positional_arguments.py")
+    )]
     #[test_case(Rule::TooManyBranches, Path::new("too_many_branches.py"))]
     #[test_case(
         Rule::TooManyReturnStatements,
@@ -150,8 +168,10 @@ mod tests {
     )]
     #[test_case(Rule::UselessElseOnLoop, Path::new("useless_else_on_loop.py"))]
     #[test_case(Rule::UselessImportAlias, Path::new("import_aliasing.py"))]
+    #[test_case(Rule::UselessImportAlias, Path::new("import_aliasing_2/__init__.py"))]
     #[test_case(Rule::UselessReturn, Path::new("useless_return.py"))]
     #[test_case(Rule::UselessWithLock, Path::new("useless_with_lock.py"))]
+    #[test_case(Rule::UnreachableCode, Path::new("unreachable.py"))]
     #[test_case(
         Rule::YieldFromInAsyncFunction,
         Path::new("yield_from_in_async_function.py")
@@ -211,6 +231,8 @@ mod tests {
         Rule::BadStaticmethodArgument,
         Path::new("bad_staticmethod_argument.py")
     )]
+    #[test_case(Rule::LenTest, Path::new("len_as_condition.py"))]
+    #[test_case(Rule::MissingMaxsplitArg, Path::new("missing_maxsplit_arg.py"))]
     fn rules(rule_code: Rule, path: &Path) -> Result<()> {
         let snapshot = format!("{}_{}", rule_code.noqa_code(), path.to_string_lossy());
         let diagnostics = test_path(
@@ -225,7 +247,7 @@ mod tests {
                 ..LinterSettings::for_rule(rule_code)
             },
         )?;
-        assert_messages!(snapshot, diagnostics);
+        assert_diagnostics!(snapshot, diagnostics);
         Ok(())
     }
 
@@ -234,9 +256,9 @@ mod tests {
         let diagnostics = test_path(
             Path::new("pylint/continue_in_finally.py"),
             &LinterSettings::for_rule(Rule::ContinueInFinally)
-                .with_target_version(PythonVersion::Py37),
+                .with_target_version(PythonVersion::PY37),
         )?;
-        assert_messages!(diagnostics);
+        assert_diagnostics!(diagnostics);
         Ok(())
     }
 
@@ -252,7 +274,7 @@ mod tests {
                 ..LinterSettings::for_rule(Rule::MagicValueComparison)
             },
         )?;
-        assert_messages!(diagnostics);
+        assert_diagnostics!(diagnostics);
         Ok(())
     }
 
@@ -268,7 +290,7 @@ mod tests {
                 ..LinterSettings::for_rule(Rule::TooManyArguments)
             },
         )?;
-        assert_messages!(diagnostics);
+        assert_diagnostics!(diagnostics);
         Ok(())
     }
 
@@ -281,7 +303,7 @@ mod tests {
                 ..LinterSettings::for_rule(Rule::TooManyArguments)
             },
         )?;
-        assert_messages!(diagnostics);
+        assert_diagnostics!(diagnostics);
         Ok(())
     }
 
@@ -294,10 +316,10 @@ mod tests {
                     max_positional_args: 4,
                     ..pylint::settings::Settings::default()
                 },
-                ..LinterSettings::for_rule(Rule::TooManyPositional)
+                ..LinterSettings::for_rule(Rule::TooManyPositionalArguments)
             },
         )?;
-        assert_messages!(diagnostics);
+        assert_diagnostics!(diagnostics);
         Ok(())
     }
 
@@ -313,7 +335,7 @@ mod tests {
                 ..LinterSettings::for_rule(Rule::TooManyBranches)
             },
         )?;
-        assert_messages!(diagnostics);
+        assert_diagnostics!(diagnostics);
         Ok(())
     }
 
@@ -329,7 +351,7 @@ mod tests {
                 ..LinterSettings::for_rule(Rule::TooManyBooleanExpressions)
             },
         )?;
-        assert_messages!(diagnostics);
+        assert_diagnostics!(diagnostics);
         Ok(())
     }
 
@@ -345,7 +367,7 @@ mod tests {
                 ..LinterSettings::for_rule(Rule::TooManyStatements)
             },
         )?;
-        assert_messages!(diagnostics);
+        assert_diagnostics!(diagnostics);
         Ok(())
     }
 
@@ -361,7 +383,7 @@ mod tests {
                 ..LinterSettings::for_rule(Rule::TooManyReturnStatements)
             },
         )?;
-        assert_messages!(diagnostics);
+        assert_diagnostics!(diagnostics);
         Ok(())
     }
 
@@ -377,7 +399,7 @@ mod tests {
                 ..LinterSettings::for_rules(vec![Rule::TooManyPublicMethods])
             },
         )?;
-        assert_messages!(diagnostics);
+        assert_diagnostics!(diagnostics);
         Ok(())
     }
 
@@ -393,18 +415,44 @@ mod tests {
                 ..LinterSettings::for_rules(vec![Rule::TooManyLocals])
             },
         )?;
-        assert_messages!(diagnostics);
+        assert_diagnostics!(diagnostics);
         Ok(())
     }
 
     #[test]
-    fn unspecified_encoding_python39_or_lower() -> Result<()> {
+    fn preview_useless_import_alias() -> Result<()> {
         let diagnostics = test_path(
-            Path::new("pylint/unspecified_encoding.py"),
-            &LinterSettings::for_rule(Rule::UnspecifiedEncoding)
-                .with_target_version(PythonVersion::Py39),
+            Path::new("pylint/import_aliasing_2/__init__.py"),
+            &LinterSettings {
+                preview: PreviewMode::Enabled,
+                ..LinterSettings::for_rule(Rule::UselessImportAlias)
+            },
         )?;
-        assert_messages!(diagnostics);
+        assert_diagnostics!(diagnostics);
+        Ok(())
+    }
+
+    #[test]
+    fn import_outside_top_level_with_banned() -> Result<()> {
+        let diagnostics = test_path(
+            Path::new("pylint/import_outside_top_level_with_banned.py"),
+            &LinterSettings {
+                preview: PreviewMode::Enabled,
+                flake8_tidy_imports: flake8_tidy_imports::settings::Settings {
+                    banned_module_level_imports: vec![
+                        "foo_banned".to_string(),
+                        "pkg_banned".to_string(),
+                        "pkg.bar_banned".to_string(),
+                    ],
+                    ..Default::default()
+                },
+                ..LinterSettings::for_rules(vec![
+                    Rule::BannedModuleLevelImports,
+                    Rule::ImportOutsideTopLevel,
+                ])
+            },
+        )?;
+        assert_diagnostics!(diagnostics);
         Ok(())
     }
 }
