@@ -208,7 +208,7 @@ impl Workspace {
         let messages: Vec<ExpandedMessage> = diagnostics
             .into_iter()
             .map(|msg| ExpandedMessage {
-                code: msg.noqa_code().map(|code| code.to_string()),
+                code: msg.secondary_code().map(ToString::to_string),
                 message: msg.body().to_string(),
                 start_location: source_code.line_column(msg.start()).into(),
                 end_location: source_code.line_column(msg.end()).into(),
