@@ -1,7 +1,7 @@
 use crate::glob::{GlobFilterCheckMode, IncludeResult};
 use crate::metadata::options::{OptionDiagnostic, ToSettingsError};
 use crate::walk::{ProjectFilesFilter, ProjectFilesWalker};
-pub use db::{Db, ProjectDatabase};
+pub use db::{CheckMode, Db, ProjectDatabase, SalsaMemoryDump};
 use files::{Index, Indexed, IndexedFiles};
 use metadata::settings::Settings;
 pub use metadata::{ProjectMetadata, ProjectMetadataError};
@@ -55,6 +55,7 @@ pub fn default_lints_registry() -> LintRegistry {
 ///    it remains the same project. That's why program is a narrowed view of the project only
 ///    holding on to the most fundamental settings required for checking.
 #[salsa::input]
+#[derive(Debug)]
 pub struct Project {
     /// The files that are open in the project.
     ///
@@ -159,7 +160,7 @@ impl Project {
     /// This is a salsa query to prevent re-computing queries if other, unrelated
     /// settings change. For example, we don't want that changing the terminal settings
     /// invalidates any type checking queries.
-    #[salsa::tracked(returns(deref))]
+    #[salsa::tracked(returns(deref), heap_size=get_size2::GetSize::get_heap_size)]
     pub fn rules(self, db: &dyn Db) -> Arc<RuleSelection> {
         self.settings(db).to_rules()
     }
@@ -213,6 +214,7 @@ impl Project {
     pub(crate) fn check(
         self,
         db: &ProjectDatabase,
+        mode: CheckMode,
         mut reporter: AssertUnwindSafe<&mut dyn Reporter>,
     ) -> Vec<Diagnostic> {
         let project_span = tracing::debug_span!("Project::check");
@@ -227,7 +229,11 @@ impl Project {
                 .map(OptionDiagnostic::to_diagnostic),
         );
 
-        let files = ProjectFiles::new(db, self);
+        let files = match mode {
+            CheckMode::OpenFiles => ProjectFiles::new(db, self),
+            // TODO: Consider open virtual files as well
+            CheckMode::AllFiles => ProjectFiles::Indexed(self.files(db)),
+        };
         reporter.set_files(files.len());
 
         diagnostics.extend(
@@ -477,7 +483,7 @@ impl Project {
         let mut diagnostics: Vec<Diagnostic> = Vec::new();
 
         // Abort checking if there are IO errors.
-        let source = source_text(db.upcast(), file);
+        let source = source_text(db, file);
 
         if let Some(read_error) = source.read_error() {
             diagnostics.push(
@@ -490,9 +496,9 @@ impl Project {
             return diagnostics;
         }
 
-        let parsed = parsed_module(db.upcast(), file);
+        let parsed = parsed_module(db, file);
 
-        let parsed_ref = parsed.load(db.upcast());
+        let parsed_ref = parsed.load(db);
         diagnostics.extend(
             parsed_ref
                 .errors()
@@ -502,17 +508,13 @@ impl Project {
 
         diagnostics.extend(parsed_ref.unsupported_syntax_errors().iter().map(|error| {
             let mut error = create_unsupported_syntax_diagnostic(file, error);
-            add_inferred_python_version_hint_to_diagnostic(
-                db.upcast(),
-                &mut error,
-                "parsing syntax",
-            );
+            add_inferred_python_version_hint_to_diagnostic(db, &mut error, "parsing syntax");
             error
         }));
 
         {
             let db = AssertUnwindSafe(db);
-            match catch(&**db, file, || check_types(db.upcast(), file)) {
+            match catch(&**db, file, || check_types(*db, file)) {
                 Ok(Some(type_check_diagnostics)) => {
                     diagnostics.extend(type_check_diagnostics.into_iter().cloned());
                 }
