@@ -10,6 +10,7 @@ use crate::types::class::{ClassLiteral, ClassType, GenericAlias};
 use crate::types::function::{FunctionType, OverloadLiteral};
 use crate::types::generics::{GenericContext, Specialization};
 use crate::types::signatures::{CallableSignature, Parameter, Parameters, Signature};
+use crate::types::tuple::TupleSpec;
 use crate::types::{
     CallableType, IntersectionType, KnownClass, MethodWrapperKind, Protocol, StringLiteralType,
     SubclassOfInner, Type, TypeVarBoundOrConstraints, TypeVarInstance, UnionType,
@@ -190,16 +191,7 @@ impl Display for DisplayRepresentation<'_> {
 
                 escape.bytes_repr(TripleQuotes::No).write(f)
             }
-            Type::Tuple(tuple) => {
-                f.write_str("tuple[")?;
-                let elements = tuple.elements(self.db);
-                if elements.is_empty() {
-                    f.write_str("()")?;
-                } else {
-                    elements.display(self.db).fmt(f)?;
-                }
-                f.write_str("]")
-            }
+            Type::Tuple(specialization) => specialization.tuple(self.db).display(self.db).fmt(f),
             Type::TypeVar(typevar) => f.write_str(typevar.name(self.db)),
             Type::AlwaysTruthy => f.write_str("AlwaysTruthy"),
             Type::AlwaysFalsy => f.write_str("AlwaysFalsy"),
@@ -224,6 +216,67 @@ impl Display for DisplayRepresentation<'_> {
     }
 }
 
+impl<'db> TupleSpec<'db> {
+    pub(crate) fn display(&'db self, db: &'db dyn Db) -> DisplayTuple<'db> {
+        DisplayTuple { tuple: self, db }
+    }
+}
+
+pub(crate) struct DisplayTuple<'db> {
+    tuple: &'db TupleSpec<'db>,
+    db: &'db dyn Db,
+}
+
+impl Display for DisplayTuple<'_> {
+    fn fmt(&self, f: &mut Formatter<'_>) -> fmt::Result {
+        f.write_str("tuple[")?;
+        match self.tuple {
+            TupleSpec::Fixed(tuple) => {
+                let elements = tuple.elements_slice();
+                if elements.is_empty() {
+                    f.write_str("()")?;
+                } else {
+                    elements.display(self.db).fmt(f)?;
+                }
+            }
+
+            // Decoder key for which snippets of text need to be included depending on whether
+            // the tuple contains a prefix and/or suffix:
+            //
+            // tuple[            yyy, ...      ]
+            // tuple[xxx, *tuple[yyy, ...]     ]
+            // tuple[xxx, *tuple[yyy, ...], zzz]
+            // tuple[     *tuple[yyy, ...], zzz]
+            //       PPPPPPPPPPPP        P
+            //            SSSSSSS        SSSSSS
+            //
+            // (Anything that appears above only a P is included only if there's a prefix; anything
+            // above only an S is included only if there's a suffix; anything about both a P and an
+            // S is included if there is either a prefix or a suffix. The initial `tuple[` and
+            // trailing `]` are printed elsewhere. The `yyy, ...` is printed no matter what.)
+            TupleSpec::Variable(tuple) => {
+                if !tuple.prefix.is_empty() {
+                    tuple.prefix.display(self.db).fmt(f)?;
+                    f.write_str(", ")?;
+                }
+                if !tuple.prefix.is_empty() || !tuple.suffix.is_empty() {
+                    f.write_str("*tuple[")?;
+                }
+                tuple.variable.display(self.db).fmt(f)?;
+                f.write_str(", ...")?;
+                if !tuple.prefix.is_empty() || !tuple.suffix.is_empty() {
+                    f.write_str("]")?;
+                }
+                if !tuple.suffix.is_empty() {
+                    f.write_str(", ")?;
+                    tuple.suffix.display(self.db).fmt(f)?;
+                }
+            }
+        }
+        f.write_str("]")
+    }
+}
+
 impl<'db> OverloadLiteral<'db> {
     // Not currently used, but useful for debugging.
     #[expect(dead_code)]
@@ -307,15 +360,19 @@ pub(crate) struct DisplayGenericAlias<'db> {
 
 impl Display for DisplayGenericAlias<'_> {
     fn fmt(&self, f: &mut Formatter<'_>) -> fmt::Result {
-        write!(
-            f,
-            "{origin}{specialization}",
-            origin = self.origin.name(self.db),
-            specialization = self.specialization.display_short(
-                self.db,
-                TupleSpecialization::from_class(self.db, self.origin)
-            ),
-        )
+        if self.origin.is_known(self.db, KnownClass::Tuple) {
+            self.specialization.tuple(self.db).display(self.db).fmt(f)
+        } else {
+            write!(
+                f,
+                "{origin}{specialization}",
+                origin = self.origin.name(self.db),
+                specialization = self.specialization.display_short(
+                    self.db,
+                    TupleSpecialization::from_class(self.db, self.origin)
+                ),
+            )
+        }
     }
 }
 
